config_params = {
    'database': [
        {
            'host': 'ORACLE_HOST'
        },
        {
            'port': 'ORACLE_PORT'
        },
        {
            'user': 'ORACLE_USER'
        },
        {
            'pwd': 'ORACLE_PWD'
        },
        {
            'service': 'ORACLE_DB'
        },
        {
            'sid': 'ORACLE_SID'
        },
        {
            'schema': 'ORACLE_SCHEMA'
        }
    ],
    'kafka': [
        {
            'host': 'KAFKA_SERVER'
        },
        {
            'port': 'KAFKA_PORT'
        },
        {
            'user': 'KAFKA_USER'
        },
        {
            'pwd': 'KAFKA_PWD'
        }
    ],
    'sftp': [
        {
            'host': 'SFTP_HOST'
        },
        {
            'port': 'SFTP_PORT'
        },
        {
            'user': 'SFTP_USER'
        },
        {
            'pwd': 'SFTP_PWD'
        }
    ]
}
