# MongoDB Documents
MONGO_COLLECTION_UPLOAD_FILE_USER_ID = 'user_id'
MONGO_COLLECTION_UPLOAD_FILE_PARTNER_ID = 'partner_id'
MONGO_COLLECTION_UPLOAD_FILE_CLIENT_ID = 'partner_id'
MONGO_COLLECTION_UPLOAD_FILE = 'upload_files'
MONGO_COLLECTION_CONFIG_RULES = 'config_rules'
MONGO_COLLECTION_CONFIG_RULES_DEFINED_COLUMNS = 'config_rules_columns'
MONGO_COLLECTION_CONFIG_RULES_DEFINED_INDEX_COLUMNS = 'config_rules_columns_index'
MONGO_COLLECTION_CONFIG_RULES_DEFINED_AMOUNT_COLUMN = 'config_rules_column_amount'
MONGO_COLLECTION_CONFIG_RULES_DEFINED_DATE_COLUMN = 'config_rules_column_date'
MONGO_COLLECTION_CONFIG_RULES_DEFINED_MAPPING_COLUMNS = 'config_rules_columns_mapping'
MONGO_COLLECTION_PROC_RECONCILED_CNF = 'config_process'
MONGO_COLLECTION_PROC_RECONCILED_FILE = 'config_process_files'
MONGO_COLLECTION_DATA_RECONCILED = 'reconciled'
MONGO_COLLECTION_DATA_TOTAL_RECONCILED = 'total_reconciled'
MONGO_COLLECTION_DATA_DUPLICATES = 'duplicates'
MONGO_COLLECTION_DATA_TOTAL_DUPLICATES = 'total_duplicates'
MONGO_COLLECTION_DATA_UNRECONCILED = 'unreconciled'
MONGO_COLLECTION_DATA_TOTAL_UNRECONCILED = 'total_unreconciled'
MONGO_COLLECTION_DATA_USERS = 'users'
MONGO_COLLECTION_DATA_ROLES = 'roles'
MONGO_COLLECTION_DATA_PARTNERS = 'partners'
MONGO_COLLECTION_DATA_TYPE_CYCLES = 'type_cycles'
MONGO_COLLECTION_DATA_CYCLES = 'cycles'
MONGO_COLLECTION_DATA_PROG_SCHEDULERS = 'program_scheduler'
MONGO_COLLECTION_AUDITORY_LOGS = 'auditory_logs'
MONGO_COLLECTION_DATA_CONFIG_PARAMS = 'config_params'

MONGO_COLLECTIONS = {
    MONGO_COLLECTION_DATA_RECONCILED,
    MONGO_COLLECTION_DATA_TOTAL_RECONCILED,
    MONGO_COLLECTION_DATA_UNRECONCILED,
    MONGO_COLLECTION_DATA_TOTAL_UNRECONCILED,
    MONGO_COLLECTION_DATA_DUPLICATES,
    MONGO_COLLECTION_DATA_TOTAL_DUPLICATES,
    MONGO_COLLECTION_CONFIG_RULES,
    MONGO_COLLECTION_CONFIG_RULES_DEFINED_COLUMNS,
    MONGO_COLLECTION_CONFIG_RULES_DEFINED_INDEX_COLUMNS,
    MONGO_COLLECTION_CONFIG_RULES_DEFINED_AMOUNT_COLUMN,
    MONGO_COLLECTION_CONFIG_RULES_DEFINED_DATE_COLUMN,
    MONGO_COLLECTION_CONFIG_RULES_DEFINED_MAPPING_COLUMNS,
    MONGO_COLLECTION_UPLOAD_FILE,
    MONGO_COLLECTION_PROC_RECONCILED_CNF,
    MONGO_COLLECTION_PROC_RECONCILED_FILE,
    MONGO_COLLECTION_DATA_USERS,
    MONGO_COLLECTION_DATA_ROLES,
    MONGO_COLLECTION_DATA_PARTNERS,
    MONGO_COLLECTION_DATA_TYPE_CYCLES,
    MONGO_COLLECTION_DATA_CYCLES,
    MONGO_COLLECTION_DATA_PROG_SCHEDULERS,
    MONGO_COLLECTION_AUDITORY_LOGS,
    MONGO_COLLECTION_DATA_CONFIG_PARAMS
}